@@ -1,17 +1,16 @@
 """
 Basic functions for getting data from a dWeather gateway via https.
 """
-import requests, datetime, io, gzip
+import os, pickle, math, requests, datetime, io, gzip, json, csv
 from dweather_client.ipfs_errors import *
-from dweather_client.utils import listify_period, celcius_to_fahrenheit
+from dweather_client.utils import listify_period, lat_lon_to_rtma_grid, find_closest_lat_lon, build_rtma_reverse_lookup, build_rtma_lookup, conventional_lat_lon_to_cpc, celcius_to_fahrenheit
 import dweather_client.ipfs_datasets
-import csv
+from collections import deque
 
 MM_TO_INCHES = 0.0393701
 RAINFALL_PRECISION = 5
 GATEWAY_URL = 'https://gateway.arbolmarket.com'
 
-
 def get_heads(url=GATEWAY_URL):
     """
     Get heads.json for a given IPFS gateway.
@@ -32,7 +31,6 @@ def get_heads(url=GATEWAY_URL):
     r.raise_for_status()
     return r.json()
 
-
 def get_metadata(hash_str, url=GATEWAY_URL):
     """
     Get the metadata file for a given hash.
@@ -61,13 +59,56 @@ def get_metadata(hash_str, url=GATEWAY_URL):
             'year delimiter': '\n'
         }
     """
-    metadata_url = url + "/ipfs/" + hash_str + "/metadata.json"
+    metadata_url = "%s/ipfs/%s/metadata.json" % (url, hash_str)
     r = requests.get(metadata_url)
     r.raise_for_status()
     return r.json()
 
+def traverse_ll(head):
+    release_itr = head
+    release_ll = deque()
+    while True:
+        release_ll.appendleft(release_itr)
+        prev_release = get_metadata(release_itr)['previous hash']
+        if prev_release != None:
+            release_itr = prev_release
+        else:
+            return release_ll
 
-def get_station_csv(station_id):
+
+def get_hurricane_release_dict(release_hash, url=GATEWAY_URL):
+    url = "%s/ipfs/%s/history.json.gz" % (url, release_hash)
+    resp = requests.get(url)
+    resp.raise_for_status()
+    with gzip.GzipFile(fileobj=io.BytesIO(resp.content)) as zip_data:
+        return json.loads(zip_data.read().decode("utf-8"))
+
+def get_hurricane_dict(head=get_heads()['atcf_btk-seasonal']):
+    """
+    Get a hurricane dictionary for the atcf_btk-seasonal dataset. 
+
+    To get a unique value to query the dict by storm, use BASIN + CY + the year
+    part of the HOUR value. BASIN is the ocean, CY is the storm index, and
+    the year is needed as well because the storm index resets every year.
+
+    Note that there will be multiple readings with the same HOUR value,
+    as readings are taken more than once per hour and then rounded to the nearest
+    hour before posting. 
+    """
+    heads = get_heads()
+    hurr_head = heads['atcf_btk-seasonal']
+    release_ll = traverse_ll(hurr_head)
+    hurr_dict = {}
+    for release_hash in release_ll:
+        release_content = get_hurricane_release_dict(release_hash)
+        try:
+            hurr_dict['features'].append(release_content['features'])
+        except KeyError:
+            hurr_dict.update(release_content)
+    return hurr_dict
+
+
+def get_station_csv(station_id, url=GATEWAY_URL):
     """
     Retrieve the contents of a station data csv file.
     Args:
@@ -77,16 +118,12 @@ def get_station_csv(station_id):
     """
     all_hashes = get_heads()
     dataset_hash = all_hashes["ghcnd"]
-    dataset_url = GATEWAY_URL + "/ipfs/" + dataset_hash + '/' + station_id + ".csv.gz"
-    print(dataset_url)
+    dataset_url = "%s/ipfs/%s/%s.csv.gz" % (url, dataset_hash, str(station_id))
     r = requests.get(dataset_url)
-    print(r)
     r.raise_for_status()
     with gzip.GzipFile(fileobj=io.BytesIO(r.content)) as zip_data:
-        print(zip_data)
         return zip_data.read().decode("utf-8")
 
-
 def parse_station_temps_as_dict(csv_text, use_fahrenheit=True):
     """
     Parse a station CSV file and get column values
@@ -121,23 +158,19 @@ def parse_station_temps_as_dict(csv_text, use_fahrenheit=True):
 
     return tmins, tmaxs
 
-
-
 def get_station_by_wmo_id(wmo_id):
     pass
 
 def get_station_by_airport_code(code):
     pass
 
-
-def get_hash_cell(hash_str, coord_str):
-    dataset_url = GATEWAY_URL + '/ipfs/' + hash_str + '/' + coord_str
+def get_hash_cell(hash_str, coord_str, url=GATEWAY_URL):
+    dataset_url = '%s/ipfs/%s/%s' % (url, hash_str, coord_str)
     r = requests.get(dataset_url)
     r.raise_for_status()
     return r.text
 
-
-def get_zipped_hash_cell(url, hash_str, coord_str):
+def get_zipped_hash_cell(hash_str, coord_str, url=GATEWAY_URL):
     """
     Read a text file on the ipfs server compressed with gzip.
     Args:
@@ -147,12 +180,64 @@ def get_zipped_hash_cell(url, hash_str, coord_str):
     Returns:
         the contents of the file as a string
     """
-    dataset_url = url + "/ipfs/" + hash_str + '/' + coord_str + ".gz"
+    dataset_url = '%s/ipfs/%s/%s.gz' % (url, hash_str, coord_str)
     r = requests.get(dataset_url)
     r.raise_for_status()
     with gzip.GzipFile(fileobj=io.BytesIO(r.content)) as zip_data:
         return zip_data.read().decode("utf-8")
 
+def get_best_rtma_dict(lat, lon):
+    lat, lon = float(lat), float(lon)
+    if ((lat < 20) or (53 < lat)):
+        raise InputOutOfRangeError('RTMA only covers latitudes 20 thru 53')
+    if ((lon < 228) or (300 < lon)):
+        raise InputOutOfRangeError('RTMA only covers longitudes -132 thru -60')
+    lat, lon = conventional_lat_lon_to_cpc(lat, lon)
+    lat, lon = str(lat), str(lon)
+    rtma_head = get_heads()['rtma_pcp-hourly']
+    metadata = get_metadata(rtma_head)
+    r = requests.get('%s/ipfs/%s/grid_history.txt.gz' % (GATEWAY_URL, rtma_head))
+    r.raise_for_status()
+    with gzip.GzipFile(fileobj=io.BytesIO(r.content)) as grid_history_file:
+        grid_history = grid_history_file.read().decode('utf-8')
+    r_lookup = build_rtma_reverse_lookup(grid_history)
+    new_grid = list(r_lookup.keys())[-1]
+    valid_lat_lons = pickle.load(open(os.path.join(os.path.dirname(__file__), 'etc/rtma_lat_lons.p'), 'rb'))
+    closest_lat_lon = find_closest_lat_lon(valid_lat_lons, (lat, lon))
+    lat_xy = r_lookup[new_grid]['lat'][closest_lat_lon[0]]
+    lon_xy = r_lookup[new_grid]['lon'][closest_lat_lon[1]]
+    assert lat_xy == lon_xy
+    return get_rtma_dict(lat_xy[0], lat_xy[1], grid_history)
+    """
+    print("original: " + lat + " " + lon)
+    print("closest valid lat lon: " + str(closest_lat_lon))
+    print("xy from lat lookup: " + str(lat_xy))
+    print("xy from lon lookup: " + str(lon_xy))
+    print("lat from reverse lookup of xy: " + str(lookup[new_grid][0][lat_xy[1]][lat_xy[0]]))
+    print("lon from reverse lookup of xy: "+ str(lookup[new_grid][1][lon_xy[1]][lon_xy[0]]))
+"""
+def get_rtma_dict(x, y, grid_history=None):
+    rtma_head = get_heads()['rtma_pcp-hourly']
+    metadata = get_metadata(rtma_head)
+    if grid_history is None:
+        r = requests.get('%s/ipfs/%s/grid_history.txt.gz' % (GATEWAY_URL, rtma_head))
+        r.raise_for_status()
+        with gzip.GzipFile(fileobj=io.BytesIO(r.content)) as grid_history_file:
+            grid_history = grid_history_file.read().decode('utf-8')
+    r = requests.get('%s/ipfs/%s/%s_%s.gz' % (GATEWAY_URL, rtma_head, str(x).zfill(4), str(y).zfill(4)))
+    r.raise_for_status()
+    with gzip.GzipFile(fileobj=io.BytesIO(r.content)) as cell_text_file:
+        cell_text = cell_text_file.read().decode('utf-8') 
+    rtma_start_date = datetime.datetime.strptime(metadata['date range'][0], "%Y-%m-%dT%H:%M:%S")
+    rtma_end_date = datetime.datetime.strptime(metadata['date range'][1], "%Y-%m-%dT%H:%M:%S")
+    hour_itr = rtma_start_date
+    rtma_dict = {}
+    for year_data in cell_text.split('\n'):
+        for hour_data in year_data.split(','):
+            rtma_dict[hour_itr] = hour_data
+            hour_itr = hour_itr + datetime.timedelta(hours=1)
+    assert hour_itr == rtma_end_date
+    return rtma_dict
 
 def get_dataset_cell(lat, lon, dataset_revision):
     """ 
@@ -183,14 +268,13 @@ def get_dataset_cell(lat, lon, dataset_revision):
     coord_str = "{:.3f}_{:.3f}".format(lat,lon)
     try:
         if "compression" in metadata and metadata["compression"] == "gzip":
-            text_data = get_zipped_hash_cell(GATEWAY_URL, dataset_hash, coord_str)
+            text_data = get_zipped_hash_cell(dataset_hash, coord_str)
         else:
             text_data = get_hash_cell(dataset_hash, coord_str)
         return metadata, text_data
     except requests.exceptions.RequestException as e:
         raise CoordinateNotFoundError('Coordinate ({}, {}) not found  on ipfs in dataset revision {}'.format(lat, lon, dataset_revision))
 
-
 def get_rainfall_dict(lat, lon, dataset_revision, return_metadata=False):
     """ 
     Build a dict of rainfall data for a given grid cell.
@@ -224,7 +308,6 @@ def get_rainfall_dict(lat, lon, dataset_revision, return_metadata=False):
     else:
         return rainfall_dict
 
-
 def get_rev_rainfall_dict(lat, lon, dataset, desired_end_date, latest_rev):
     """
     Build a dictionary of rainfall data. Include as much of the most accurate, final data as possible. Start by buidling from the most accurate data,
@@ -261,7 +344,6 @@ def get_rev_rainfall_dict(lat, lon, dataset, desired_end_date, latest_rev):
     # If we don't reach the desired dataset, return all data.
     return all_rainfall, is_final
 
-
 def get_temperature_dict(lat, lon, dataset_revision, return_metadata=False):
     """
     Build a dict of temperature data for a given grid cell.
@@ -298,7 +380,6 @@ def get_temperature_dict(lat, lon, dataset_revision, return_metadata=False):
     else:
         return highs, lows
 
-
 def get_rev_temperature_dict(lat, lon, dataset, desired_end_date, latest_rev):
     """
     Build a dictionary of rainfall data. Include as much final data as possible. If the desired end date
@@ -336,7 +417,6 @@ def get_rev_temperature_dict(lat, lon, dataset, desired_end_date, latest_rev):
     # If we don't reach the desired dataset, return all data.
     return highs, lows, is_final
 
-
 def get_rev_tagged_temperature_dict(lat, lon, dataset, desired_end_date=None):
     ''' Build temps with a revision tag by each date
     Args:
@@ -363,5 +443,3 @@ def get_rev_tagged_temperature_dict(lat, lon, dataset, desired_end_date=None):
 
     # If we don't reach the desired dataset, return all data.
     return highs, lows
-
-
