@@ -1,6 +1,9 @@
 from dweather_client import http_client, utils
 from datetime import date
 
+def test_get_best_rtma_dict():
+    http_client.get_best_rtma_dict('49.01', '235.01')
+
 def test_http_client_rainfall():
     dataset = 'chirps_05'
     dataset_revision = 'chirps_05-daily'
